@@ -1,5 +1,7 @@
 # pkgdown::build_site()
-# R CMD check --as-cran PRECAST_1.2.tar.gz
+# build_article(name="PRECAST.DLPFC") # Solely compile one article for updating.
+# build_article(name="PRECAST.BreastCancer")
+# R CMD check --as-cran PRECAST_1.3.tar.gz
 # devtools::check_win_release()
 # iDR.SC <- function(...) UseMethod("iDR.SC")
 model_set <- function(Sigma_equal=FALSE, Sigma_diag=TRUE,mix_prop_heter=TRUE,
@@ -61,12 +63,12 @@ ICM.EM <- function(XList, q, K, AdjList=NULL,  Adjlist_car=NULL, posList = NULL,
           ", Sigma_diag=", Sigma_diag, ', mix_prop_heter=', mix_prop_heter)
   
   if(is.null(AdjList) && !is.null(posList))
-    AdjList <- lapply(posList, getAdj, platform=platform)
+    AdjList <- getAdjList(posList, platform=platform)
   if(any(sapply(AdjList, nrow)!=sapply(XList, nrow)))  
     stop('The dimension of Adj matrix does not match that of X!\n')
   
   if(is.null(Adjlist_car)) Adjlist_car <- AdjList
-  #XList <- lapply(XList, scale, scale=scale_flag)
+  
   Xmat <- NULL
   for(r in 1:r_max){
     Xmat <- rbind(Xmat, XList[[r]])
@@ -178,12 +180,12 @@ idrsc <- function(XList, q, K, AdjList=NULL,  Adjlist_car=NULL, posList = NULL,
           ", Sigma_diag=", Sigma_diag, ', mix_prop_heter=', mix_prop_heter)
   
   if(is.null(AdjList) && !is.null(posList))
-    AdjList <- lapply(posList, getAdj, platform=platform)
+    AdjList <- getAdjList(posList,  platform=platform)
   if(any(sapply(AdjList, nrow)!=sapply(XList, nrow)))  
     stop('The dimension of Adj matrix does not match that of X!\n')
   
   if(is.null(Adjlist_car)) Adjlist_car <- AdjList
-  #XList <- lapply(XList, scale, scale=scale_flag)
+ 
   Xmat <- NULL
   for(r in 1:r_max){
     Xmat <- rbind(Xmat, XList[[r]])
@@ -624,7 +626,16 @@ wpca <- function(X, q, weighted=T){
   return(out)
 }
 
-
+getAdjList <- function(posList, platform='Visium', ...){
+  
+  if(!inherits(posList, "list")) stop("posList must be a list consisting of matrix.")
+  if(tolower(platform) %in% c("st", "visium")){
+      AdjList <- pbapply::pblapply(posList, getAdj_reg, platform=platform)
+  }else{
+      AdjList <- pbapply::pblapply(posList, function(x, ...)getAdj_auto(x, ...))
+  }
+  return(AdjList)
+}
 
 
 
